// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_messages.dataserver;

/// Snippet selector
///
/// A snippet is a piece of data with a specific data class. This data snippet
/// can be retrieved from any table that contains a field that matches this
/// snippet. You can also force an origin database and table.
class SnippetSelector {
  /// Origin database
  final String database;

  /// Origin table
  final String table;

  /// Retrieve all matching fields
  final bool all;

  /// Desired field data class
  final String dataClass;

  /// Desired field attributes
  final Map<String, String> attributes;

  /// Input parameters
  final Map<String, String> parameters;

  /// Constructor
  SnippetSelector(this.dataClass,
      {this.database: '',
      this.table: '',
      this.all: false,
      this.attributes: const {},
      this.parameters: const {}});

  /// Construct from PDS.
  factory SnippetSelector.fromPds(Map<String, dynamic> data) {
    return new SnippetSelector(data['class'] as String,
        database: data['database'] as String,
        table: data['table'] as String,
        all: data['all'] as bool,
        attributes: data['attributes'] as Map<String, String>,
        parameters: data['parameters'] as Map<String, String>);
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {
      'database': database,
      'table': table,
      'class': dataClass,
      'all': all,
      'attributes': pdsFromMap(attributes),
      'parameters': pdsFromMap(parameters)
    };
  }
}

/// Field data
class FieldData {
  /// Field data class
  final String dataClass;

  /// Field value
  final dynamic value;

  /// Constructor
  FieldData(this.dataClass, this.value);

  /// Construct from PDS.
  factory FieldData.fromPds(Map<String, dynamic> data) {
    return new FieldData(data['class'] as String, data['value']);
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {'class': dataClass, 'value': pdsFromDynamic(value)};
  }
}

/// Snippet data
class SnippetData {
  /// Origin database
  final String database;

  /// Origin table
  final String table;

  /// Data
  final FieldData data;

  /// Constructor
  SnippetData(this.database, this.table, this.data);

  /// Construct from PDS.
  factory SnippetData.fromPds(Map<String, dynamic> data) {
    return new SnippetData(data['database'] as String, data['table'] as String,
        new FieldData.fromPds(data['data'] as Map<String, dynamic>));
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {'database': database, 'table': table, 'data': data.toPds()};
  }
}
